#include "copyright.h"
#include "port.h"

/*
 * application main
 *
 */

#include "rpcApp.h"

extern long UserMain();

RPCMain(argc, argv)
int argc;
char **argv;
{
    RPCFunction *CommandArray;		/* menu/function array */
    RPCSpot spot;
    lsList cmdList;
    long userOptionWord;
    char *display;
    long size;
    static int been_here_before = 0;            /* for 'wbaker' */

#ifndef aiws
    /*
     * ignore stop signals, it's real bad to stop remote applications...
     * so I won't let you (note that AIX does not support these)
     */
    (void) signal(SIGSTOP, SIG_IGN);
    (void) signal(SIGTSTP, SIG_IGN);
#endif

    if (been_here_before == 1) {
	(void) fprintf(stderr, "You've called a NULL function pointer!!!!!\n");
	(void) fprintf(stderr, "This has been a public service announcement from\n");
	(void) fprintf(stderr, "your friendly paranoid rpc library\n");
	exit(RPC_BAD_EXIT);
    }

    been_here_before = 1;

    errProgramName(argv[0]);

    if (argc == 10) {

	/* determine the application is byte swapped relative to the server */

	RPCByteSwappedApplication(atoi(argv[5]),
					 atoi(argv[6]),
					 atoi(argv[7]),
					 atoi(argv[8]),
					 atoi(argv[9]));
	    
	/* host port protocol */
	if (RPCConnectToServer(argv[2], atoi(argv[3]), argv[4],
					&RPCSendStream,
					&RPCReceiveStream) == RPC_ERROR) {
	    (void) fprintf(stderr, "RPC Error: can not connect to the server\n");
	    exit(RPC_BAD_EXIT);
	}

    } else {
	(void) fprintf(stderr, "RPC Error: incorrect number of arguments to remote application\n");
	exit(RPC_BAD_EXIT);
    }

    if (vemInitializeApplication(&display, &spot, &cmdList, &userOptionWord)
	!= OCT_OK) {
	(void) fprintf(stderr, "RPC Error: application: error in initialization\n");
	exit(RPC_BAD_EXIT);
    }

    /*
     * Since rpcSignal sends SIGUSR1 by default, make sure that the
     * RPC application does not die upon receving it. The user can reset 
     * this if so desired.
     */
    (void) signal( SIGUSR1, SIG_IGN );	/*  */

    if ((size = UserMain(display, &spot, cmdList, userOptionWord, &CommandArray)) < 0) {
	(void) fprintf(stderr, "RPC Error: UserMain returned a value less than zero\n");
	exit(RPC_BAD_EXIT);
    }
    
    if (RPCApplicationFunctionComplete() != RPC_OK) {
	exit(RPC_BAD_EXIT);
    }

    /*
     * tell VEM about the remote user functions
     */

    if (vemSendMenu(CommandArray, size) != OCT_OK) {
	exit(RPC_BAD_EXIT);
    }
    
    /* process VEM remote user function requests */
    if (RPCApplicationProcessEvents(CommandArray, size) != RPC_OK) {
	exit(RPC_BAD_EXIT);
    }

    return RPC_GOOD_EXIT;
}


#ifndef lint
/*
 * XXX this is to make sure appOct.c (and all of the Oct routines)
 * are brought in before any routines for 'liboct.a'
 */
static void (*dummy)() = octBegin;
#endif

