/*
 * hilite terminals, bags, instances and nets given a name
 */

#include "copyright.h"
#include "port.h"
#include "rpc.h"

vemSelSet set = 0;

#define NIL(type)  ((type *) 0)

char *optProgName = "show";

/*ARGSUSED*/
int
showExit(spot, arglist, uow)
RPCSpot *spot;
lsList arglist;
long uow;
{
    (void) vemClearSelSet(set);
    RPCExit(0);
}


/*ARGSUSED*/
int
showClear(spot, arglist, uow)
RPCSpot *spot;
lsList arglist;
long uow;
{
    if (set != 0) {
	(void) vemClearSelSet(set);
    }
    vemPrompt();
    return(RPC_OK);
}


/*ARGSUSED*/
int
showNet(spot, arglist, uow)
RPCSpot *spot;
lsList arglist;
long uow;
{
    RPCArg *arg;
    octObject facet, net;
    Pointer pointer;
    

    if (lsLength(arglist) != 1) {
	VEMMSG("\"netname\" : ShowNet\n");
	vemPrompt();
	return(RPC_OK);
    }	

    (void) lsDelBegin(arglist, &pointer);
    arg = (RPCArg *) pointer;
    
    if (arg->argType != VEM_TEXT_ARG) {
	VEMMSG("\"netname\" : ShowNet\n");
	vemPrompt();
	return(RPC_OK);
    }	

    facet.type = OCT_FACET;
    facet.objectId = spot->facet;
    if (octGetById(&facet) < OCT_OK) {
	octError("ShowNet: Facet Id");
	return(RPC_OK);
    }

    net.type = OCT_NET;
    net.contents.net.name = arg->argData.string;
    if (octGetByName(&facet, &net) < OCT_OK) {
	octError("ShowNet");
	return(RPC_OK);
    }
	
    if (set == 0) {
	set = vemNewSelSet(facet.objectId, 0, 999, 0, 1, 1, 4, 4, "0000 0110 0110 0000");
    }

    (void) vemAddSelSet(set, net.objectId);

    vemPrompt();

    return(RPC_OK);
}


/*ARGSUSED*/
int
showTerm(spot, arglist, uow)
RPCSpot *spot;
lsList arglist;
long uow;
{
    RPCArg *arg;
    Pointer pointer;
    octObject facet, term;

    
    if (lsLength(arglist) != 1) {
	VEMMSG("\"termname\" : ShowTerm\n");
	vemPrompt();
	return(RPC_OK);
    }	

    (void) lsDelBegin(arglist, &pointer);
    arg = (RPCArg *) pointer;
    
    if (arg->argType != VEM_TEXT_ARG) {
	VEMMSG("\"termname\" : ShowTerm\n");
	vemPrompt();
	return(RPC_OK);
    }	

    facet.type = OCT_FACET;
    facet.objectId = spot->facet;
    if (octGetById(&facet) < OCT_OK) {
	octError("ShowTerm: Facet Id");
	return(RPC_OK);
    }

    term.type = OCT_TERM;
    term.contents.term.name = arg->argData.string;
    if (octGetByName(&facet, &term) < OCT_OK) {
	octError("ShowTerm");
	return(RPC_OK);
    }
	
    if (set == 0) {
	set = vemNewSelSet(facet.objectId, 0, 999, 0, 1, 1, 4, 4, "0000 0110 0110 0000");
    }

    (void) vemAddSelSet(set, term.objectId);

    vemPrompt();
    
    return(RPC_OK);
}


/*ARGSUSED*/
int
showInst(spot, arglist, uow)
RPCSpot *spot;
lsList arglist;
long uow;
{
    RPCArg *arg;
    Pointer pointer;
    octObject facet, inst;


    if (lsLength(arglist) != 1) {
	VEMMSG("\"instname\" : ShowInst\n");
	vemPrompt();
	return(RPC_OK);
    }	

    (void) lsDelBegin(arglist, &pointer);
    arg = (RPCArg *) pointer;
    
    if (arg->argType != VEM_TEXT_ARG) {
	VEMMSG("\"instname\" : ShowInst\n");
	vemPrompt();
	return(RPC_OK);
    }	

    facet.type = OCT_FACET;
    facet.objectId = spot->facet;
    if (octGetById(&facet) < OCT_OK) {
	octError("ShowInst: Facet Id");
	return(RPC_OK);
    }

    inst.type = OCT_INSTANCE;
    inst.contents.instance.name = arg->argData.string;
    if (octGetByName(&facet, &inst) < OCT_OK) {
	octError("ShowInst");
	return(RPC_OK);
    }
	
    if (set == 0) {
	set = vemNewSelSet(facet.objectId, 0, 999, 0, 1, 1, 4, 4, "0000 0110 0110 0000");
    }

    (void) vemAddSelSet(set, inst.objectId);

    vemPrompt();

    return(RPC_OK);
}


/*ARGSUSED*/
int
showBag(spot, arglist, uow)
RPCSpot *spot;
lsList arglist;
long uow;
{
    RPCArg *arg;
    Pointer pointer;
    octObject facet, bag;

    if (lsLength(arglist) != 1) {
	VEMMSG("\"bagname\" : ShowBag\n");
	vemPrompt();
	return(RPC_OK);
    }	

    (void) lsDelBegin(arglist, &pointer);
    arg = (RPCArg *) pointer;
    
    if (arg->argType != VEM_TEXT_ARG) {
	VEMMSG("\"bagname\" : ShowBag\n");
	vemPrompt();
	return(RPC_OK);
    }	

    facet.type = OCT_FACET;
    facet.objectId = spot->facet;
    if (octGetById(&facet) < OCT_OK) {
	octError("ShowBag: Facet Id");
	return(RPC_OK);
    }

    bag.type = OCT_BAG;
    bag.contents.bag.name = arg->argData.string;
    if (octGetByName(&facet, &bag) < OCT_OK) {
	octError("ShowBag");
	return(RPC_OK);
    }
	
    if (set == 0) {
	set = vemNewSelSet(facet.objectId, 0, 999, 0, 1, 1, 4, 4, "0000 0110 0110 0000");
    }

    (void) vemAddSelSet(set, bag.objectId);

    vemPrompt();

    return(RPC_OK);
}


RPCFunction CommandArray[] = {
    {showNet, "Show", "ShowNet", "N", 0},
    {showTerm, "Show", "ShowTerm", "T", 0},
    {showInst, "Show", "ShowInst", "I", 0},
    {showBag, "Show", "ShowBag", "B", 0},
    {showClear, "Show", "Clear", NIL(char), 0},
    {showExit, "Show", "Exit", "E", 0}
};


/*ARGSUSED*/
int
UserMain(display, spot, arglist, uow, array)
char *display;
RPCSpot *spot;
lsList arglist;
long uow;
RPCFunction **array;
{
    *array = CommandArray;
    VEMMSG("Show Initialized\n");
    vemPrompt();
    return(sizeof(CommandArray)/sizeof(struct RPCFunction));
}

