#include "copyright.h"
#include "port.h"
#include "rpc.h"

char *optProgName = "testrpc";

/*
 * variables used between remote functions
 *   not needed for normal RPC application
 */
vemSelSet set = 0;

char ubuffer[2048];

#define MYNIL(type)  (type *) 0

char helpString[] = "This is a help message for use in a dialog box.\n\
It is used to explain the purpose of the dialog\n\
box and what options the user has.";

#define MIN(a,b)     ((a) < (b) ? (a) : (b))
#define MAX(a,b)     ((a) > (b) ? (a) : (b))

/*ARGSUSED*/  
int
testOpen(spot, cmdList, userOptionWord)
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    octObject facet;

    facet.objectId = spot->facet;
    if (octGetById(&facet) < OCT_OK) {
        octError("get by id on facet");
	RPCExit(-1);
    }

    (void) sprintf(ubuffer, "UOW is %d\n", userOptionWord);
    VEMMSG(ubuffer);
       
    (void) vemOpenWindow(&facet, MYNIL(char));

    vemPrompt();

    return(RPC_OK);
}


/*ARGSUSED*/
int
testVemMessage(spot, cmdList, userOptionWord)
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    RPCArg *arg;
    Pointer pointer;
    

    if (lsLength(cmdList) != 1) {
	VEMMSG("\"string\" vem-message\n");
	vemPrompt();
	return(RPC_OK);
    }
    
    (void) sprintf(ubuffer, "UOW is %d\n", userOptionWord);
    VEMMSG(ubuffer);
       
    (void) lsFirstItem(cmdList, &pointer, (lsHandle *) 0);
    arg = (RPCArg *) pointer;
    
    if (arg->argType != VEM_TEXT_ARG) {
	VEMMSG("\"string\" vem-message\n");
	vemPrompt();
	return(RPC_OK);
    }
    VEMMSG("testrpc: ");
    VEMMSG(arg->argData.string);
    VEMMSG("\n");

    vemPrompt();

    return(RPC_OK);
}


int
testVemCommand(spot, cmdList, userOptionWord)
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    RPCArg *arg;
    Pointer pointer;
    

    if (lsLength(cmdList) != 1) {
	VEMMSG("\"command\" vem-command\n");
	vemPrompt();
	return(RPC_OK);
    }
    
    (void) lsDelEnd(cmdList, &pointer);
    arg = (RPCArg *) pointer;

    (void) sprintf(ubuffer, "UOW is %d\n", userOptionWord);
    VEMMSG(ubuffer);
       
    if (arg->argType != VEM_TEXT_ARG) {
	VEMMSG("[args] \"command\" vem-command\n");
	vemPrompt();
	return(RPC_OK);
    }

    (void) vemCommand(arg->argData.string, spot, cmdList, userOptionWord);
    vemPrompt();

    return(RPC_OK);
}


/*ARGSUSED*/
int
testSelectBySpot(spot, cmdList, userOptionWord)
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    octObject facet, item;
    RPCArg *arg;

    if (lsLength(cmdList) != 0) {
	VEMMSG(": select-by-spot\n");
	vemPrompt();
	return(RPC_OK);
    }
    
    (void) sprintf(ubuffer, "UOW is %d\n", userOptionWord);
    VEMMSG(ubuffer);
       
    facet.objectId = spot->facet;

    if (octGetById(&facet) < OCT_OK) {
	octError("facet");
	return(RPC_OK);
    }

    if (vuFindSpot(spot, &item, OCT_INSTANCE_MASK) == VEM_OK) {
	if (set == 0) {
	    set = vemNewSelSet(facet.objectId, 0, 65000, 0, 1, 1,
				   4, 4, "0001 0010 0100 1000");
	}
	(void) vemAddSelSet(set, item.objectId);
	(void) vemZoomSelSet(set, spot->theWin, 0.3);
    }

    vemPrompt();

    return(RPC_OK);
}


/*ARGSUSED*/
int
testDeselectByName(spot, cmdList, userOptionWord)
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    octObject facet, item;
    RPCArg *arg;
    Pointer pointer;
    

    (void) sprintf(ubuffer, "UOW is %d\n", userOptionWord);
    VEMMSG(ubuffer);
       
    if (lsLength(cmdList) != 1) {
	VEMMSG("\"string\" deselect-by-name\n");
	vemPrompt();
	return(RPC_OK);
    }
    
    (void) lsFirstItem(cmdList, &pointer, (lsHandle *) 0);
    arg = (RPCArg *) pointer;
    
    if (arg->argType != VEM_TEXT_ARG) {
	VEMMSG("\"string\" deselect-by-name\n");
	vemPrompt();
	return(RPC_OK);
    }

    facet.objectId = spot->facet;

    if (octGetById(&facet) < OCT_OK) {
	octError("facet");
	return(RPC_OK);
    }

    item.type = OCT_INSTANCE;
    item.contents.instance.name = arg->argData.string;
    if (octGetByName(&facet, &item) < OCT_OK) {
	octError("instance");
	return(RPC_OK);
    }

    (void) vemDelSelSet(set, item.objectId);

    vemPrompt();

    return(RPC_OK);
}


/*ARGSUSED*/
int
testRegionSelect(spot, cmdList, userOptionWord)
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    regObjGen gen;
    octObject facet, object;
    RPCArg *arg;
    Pointer pointer;
    struct octBox box;
    int count = 0;

    
    (void) sprintf(ubuffer, "UOW is %d\n", userOptionWord);
    VEMMSG(ubuffer);
       
    if (lsLength(cmdList) != 1) {
	VEMMSG("box region-select\n");
	vemPrompt();
	return(RPC_OK);
    }

    facet.objectId = spot->facet;

    if (octGetById(&facet) < OCT_OK) {
	return(RPC_OK);
    }

    (void) lsFirstItem(cmdList, &pointer, (lsHandle *) 0);
    arg = (RPCArg *) pointer;
    
    if (arg->argType != VEM_BOX_ARG) {
	VEMMSG("box region-select\n");
	vemPrompt();
	return(RPC_OK);
    }

    box.lowerLeft.x = MIN(arg->argData.points[0].thePoint.x,
			  arg->argData.points[1].thePoint.x);
    box.lowerLeft.y = MIN(arg->argData.points[0].thePoint.y,
			  arg->argData.points[1].thePoint.y);

    box.upperRight.x = MAX(arg->argData.points[0].thePoint.x,
			   arg->argData.points[1].thePoint.x);
    box.upperRight.y = MAX(arg->argData.points[0].thePoint.y,
			   arg->argData.points[1].thePoint.y);

    (void) fprintf(stderr, "box is (%d,%d), (%d,%d)\n",
		   box.lowerLeft.x, box.lowerLeft.y,
		   box.upperRight.x, box.upperRight.y);

    if (set == 0) {
	set = vemNewSelSet(facet.objectId, 0, 999, 0, 1, 1, 4, 4, "0000 0110 0110 0000");
    }

    gen = 0;
    if (regObjStart(&facet, &box, OCT_GEO_MASK, &gen, 1) != REG_OK) {
	VEMMSG("regObjStart failed\n");
	vemPrompt();
	return(RPC_OK);
    }

    while (regObjNext(gen, &object) == REG_OK) {
	count++;
	(void) vemAddSelSet(set, object.objectId);
    }

    (void) regObjFinish(gen);

    if (count > 0) {
	(void) vemZoomSelSet(set, spot->theWin, 0.3);
    } else {
	VEMMSG("nothing selected\n");
    }

    vemPrompt();
    
    return(RPC_OK);

}


/*ARGSUSED*/
int
testExit(spot, cmdList, userOptionWord)
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
{
    (void) sprintf(ubuffer, "UOW is %d\n", userOptionWord);
    VEMMSG(ubuffer);
       
    if (set) {
        /* always clear your selected sets before leaving the remote application */
        (void) vemClearSelSet(set);
    }

    /* call RPCExit(0) to exit your application, this does a clean up */
    RPCExit(0);
}


		  

/*
 * dummy functions for passing to Dialog Manager (dm package) functions and RPCUserIO
 *
 * not really need, could use 0 in place of 'dummy/voiddummy', but this
 * gets lint to shut up
 */
int
dummy()
{
}

void
voiddummy()
{
}


RPCFunction CommandArray[] = {
	{testOpen, "pane-1", "copy-window", "X", 10},
	{testVemMessage, "pane-1", "vem-message", "V", 11},
	{testSelectBySpot, "pane-2", "select-by-spot", MYNIL(char), 12},
	{testDeselectByName, "pane-2", "deselect-by-name", MYNIL(char), 13},
	{testRegionSelect, "pane-2", "region-select", MYNIL(char), 14},
	{testVemCommand, "pane-3", "vem-command", MYNIL(char), 15},
	{testExit, "pane-3", "exit-testrpc", "E", 16}
};
		  

/*
 * structures for the DIALOG BOXES created in UserMain
 *
 * Note: this is not needed for an RPC Application.  This is used 
 * to show what can be done in an RPC Application.
 */
dmWhichItem items[] = {{"button-one", 0, 0},
		       {"button-two", 0, 0},
		       {"button-three", 0, 0}};
		       
dmWhichItem items2[] = {{"button-one", 0, 0},
		        {"button-two", 0, 0},
		        {"button-three", 0, 0}};

dmTextItem items3[] = {{"item-1", 1, 20, "<no-value>", 0},
		       {"item-2", 2, 10, "initial value", 0}};
    

#define FDMASK(s) (1 << (s))

/*ARGSUSED*/
int
UserMain(display, spot, cmdList, userOptionWord, array)
char *display;
RPCSpot *spot;
lsList cmdList;
long userOptionWord;
RPCFunction *array[];
{

/* Note: only the 'return(RPC_OK)' is required for UserMain,
 * the rest is used to show what can be done in an RPC application.
 */

    char buffer[1024];
    char *date;
    int i, selected;
    FILE *logfile;                      /* file to log messages to   */
    octObject facet;                    /* current facet             */
    long tm;                            /* time in seconds           */
    char *name;
    int chrom;
    char def[1024];
    

    if ((logfile = RPCfopen("testrpc.log", "a")) == NULL) {
	fprintf(stderr, "can not open the log file\n");
	RPCExit(-1);
    }

    strcpy(def, "ggi-gabriel");
    fprintf(stderr, "default %s is %s\n", def, RPCXGetDefault("vem", def));

    time(&tm);
    fprintf(logfile, "\n\ntestrpc run at %s\n", ctime(&tm));
    fprintf(logfile, "display is %s\n", display);
    vemGetDisplayType(&name, &chrom);
    fprintf(logfile, "display name is %s, chromatism is %d\n", name, chrom);
    fprintf(logfile, "window is %d, facetId is %d, x,y are (%d, %d)\n",
	    spot->theWin, spot->facet,
	    spot->thePoint.x, spot->thePoint.y);
    fprintf(logfile, "number of arguments is %d\n", lsLength(cmdList));

    facet.objectId = spot->facet;
    octGetById(&facet);
    fprintf(logfile, "facet is %s:%s:%s (%s)\n\n",
	    facet.contents.facet.cell,
	    facet.contents.facet.view,
	    facet.contents.facet.facet,
	    facet.contents.facet.mode);

    fclose(logfile);

    if (lsLength(cmdList) > 0) {
	(void) testVemCommand(spot, cmdList, userOptionWord);
    }

    (void) dmMessage("Message Window",
"This is the official announcement of the release of Oct Tools 3.5.\n\
This version has a number of bug fixes, new documents, new packages,\n\
and has been tested on a larger number of machines.\n\
\n\
As always, please send all bugs/comments to 'octtools@eros.berkeley.edu'.\n\
\n\
			Rick Spickelmier\n\
			UC Berkeley\n", 0);

    (void) dmMultiWhich("RPC DM MultiWhich", sizeof(items) / sizeof(dmWhichItem),
		        items, dummy, helpString);
    
    for (i = 0; i < sizeof(items) / sizeof(dmWhichItem); i++) {
	if (items[i].flag == 1) {
	    sprintf(buffer, "%s selected\n", items[i].itemName);
	    VEMMSG(buffer);
	}
    }

    if (dmWhichOne("RPC DM Which One", sizeof(items2)/sizeof(dmWhichItem),
		   items2, &selected, dummy, MYNIL(char)) == VEM_OK) {
	
	    sprintf(buffer, "%s selected\n", items2[selected].itemName);
	    VEMMSG(buffer);
	    
    } else {
	VEMMSG("nothing selected\n");
    }
    
    (void) dmMultiText("RPC DM Multi Text", sizeof(items3) / sizeof(dmTextItem),
		       items3, MYNIL(char));

    for (i = 0; i < sizeof(items3) / sizeof(dmTextItem); i++) {
	sprintf(buffer, "%s is %s\n", items3[i].itemPrompt, items3[i].value);
	VEMMSG(buffer);
    }

    (void) vemOpenWindow(&facet, "=100x100+20+60");

    *array = CommandArray;

    VEMMSG("Testrpc initialized\n");
    vemPrompt();
    
    return(sizeof(CommandArray) / sizeof(RPCFunction));
}

